//
// エラー処理も書いてみたのだが，
// 実際のところ，エラー発生時の動作確認をしていないので，
// 本当にこれでよいのかどうか不安がある．
//
// とりあえず，gzopen，gzclose，gzread，gzwrite は便利だ．
//

// ファイルを圧縮して出力する
#include	<stdio.h>
#include	<zlib.h>
#include	<errno.h>


// 圧縮用のバッファサイズ
#define		BUF_SIZE	( 1 << 12 )		// 4,096


// 入出力用の構造体
typedef		struct CIO
{
	FILE	*in;	// 入力
	gzFile	out;	// 出力
}	CIO, *hCIO;


// 入出力の初期化
static	void	CIO_Init( hCIO h )
{
	h->in = NULL;
	h->out = NULL;
}

// 入出力を閉じる
static	int		CIO_Close( hCIO h )
{
	int		ret = 0;
	const char	*msg;

	// 入力
	if ( h->in != NULL )
	{
		if ( fclose( h->in ) )
			perror( "Input" );
	}

	// 出力
	if ( h->out != NULL )
	{
		if ( ( ret = gzclose( h->out ) ) == Z_OK )
			return	0;

		// エラーメッセージ
		msg = gzerror( h->out, &ret );
		if ( ret == Z_ERRNO )
			perror( "Output" );
		else
			fprintf( stderr, "Output: %s\n", gzerror( h->out, &ret ) );
	}

	return	ret;
}

// 入出力を開く
static	int		CIO_Open( hCIO h, const char *in, const char *out )
{
	CIO_Init( h );

	// 入力を開く
	if ( ( h->in = fopen( in, "rb" ) ) == NULL )
		perror( in );

	// 出力を閉じる
	if ( ( h->out = gzopen( out, "wb" ) ) == NULL )
	{
		if ( !errno )
			fprintf( stderr, "%s: Memory error\n", out );
		else
			perror( out );
	}

	// 面倒なのでまとめてエラー処理
	if ( h->in == NULL || h->out == NULL )
	{
		CIO_Close( h );
		return	-1;
	}

	return	0;
}

// 実際に圧縮する
static	int		CIO_Compress( hCIO h )
{
	int		ret;
	char	buf[ BUF_SIZE ];
	size_t	len;
	const char	*msg;

	// 入力
	while ( ( len = fread( buf, 1, BUF_SIZE, h->in ) ) > 0 )
	{
		// 出力
		if ( gzwrite( h->out, buf, len ) < len )
		{
			// 出力エラー
			msg = gzerror( h->out, &ret );
			if ( ret == Z_ERRNO )
				perror( "Output" );
			else
				fprintf( stderr, "Output: %s\n", gzerror( h->out, &ret ) );
		}
	}

	// 入力エラー
	if ( ferror( h->in ) )
	{
		perror( "Input" );
		return	-1;
	}

	return	0;
}

// ファイル in_path を圧縮して，ファイル out_path に出力する
int		Compress( const char *in_path, const char *out_path )
{
	int		ret;
	CIO		io;

	// 入出力を開く
	if ( CIO_Open( &io, in_path, out_path ) )
		return	-1;

	// 実際に圧縮する
	ret = CIO_Compress( &io );

	// 入出力を閉じる
	if ( !ret )
		return	CIO_Close( &io );
	CIO_Close( &io );

	return	ret;
}

int		main( int argc, char *argv[] )
{
	if ( argc != 3 )
	{
		printf( "Usage: %s Input Output\n", argv[0] );
		return	1;
	}

	return	Compress( argv[1], argv[2] );
}
