#ifndef		__ZLIB_IO_H_
#define		__ZLIB_IO_H_


// Error codes.
#define		ZIO_OK		0		// No error.
#define		ZIO_EHDL	( -1 )	// Invalid handle.
#define		ZIO_EMEM	( -2 )	// Memory error.
#define		ZIO_EIO		( -3 )	// I/O error.
#define		ZIO_EARG	( -4 )	// Invalid argument.
#define		ZIO_EFMT	( -5 )	// Invalid format.
#define		ZIO_EINV	( -6 )	// Unknown error.


#ifdef	__cplusplus
extern	"C"	{
#endif


// Handle.
typedef		struct ZIO		*hZIO;

// On memory.
typedef		struct ZMem
{
	void	*buf;	// Buffer.
	int		avail;	// Number of bytes available.
	int		size;	// Buffer size.
}	ZMem, *hZMem;


// Compress.
int		ZIO_Compress( const char *src, const char *dst );
// Decompress.
int		ZIO_Decompress( const char *src, const char *dst );

// Open (mode is [i][rw][0-9]).
//   i: Inverted mode (read and compress, decompress and write).
//  rw: Read from file or write to file.
// 0-9: Compression level (6: default, 0: faster, 9: smaller).
int		ZIO_Open( hZIO *h, const char *path, const char *mode );
// Close (flush output buffer).
int		ZIO_Close( hZIO h );

// Read (return number of bytes read).
int		ZIO_Read( hZIO h, void *buf, int n );
// Write (return number of bytes written).
int		ZIO_Write( hZIO h, const void *buf, int n );

// Load on memory (mode is passed to ZIO_Open()).
// This function reallocates memory to h->buf by realloc().
int		ZMem_Load( hZMem h, const char *path, const char *mode );
// Save to file (mode is passed to ZIO_Open()).
int		ZMem_Save( hZMem h, const char *path, const char *mode );


#ifdef	__cplusplus
}
#endif


#endif
